;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Helper procedures for working with egg files.
;;;
;;; Copyright (c) 2018, Evan Hanson
;;;
;;; See LICENSE for details.
;;;

(declare
  (module (beaker egg info))
  (export egg-dependencies
          egg-extensions
          egg-name
          egg-programs)
  (import (chicken condition)
          (chicken pathname)
          (srfi 1)))

(define (egg-name egg-file)
  (string->symbol (pathname-file egg-file)))

(define (egg-info-slot egg-info name)
  (alist-ref name egg-info eq? '()))

(define (read-egg-info egg-file)
  (handle-exceptions _ '() (with-input-from-file egg-file read)))

(define (egg-dependencies egg-file)
  (map (lambda (d) (if (pair? d) (car d) d))
       (egg-info-slot (read-egg-info egg-file) 'dependencies)))

(define (egg-components egg-file #!optional type)
  (let ((egg-info (read-egg-info egg-file)))
    (if (not type)
        (egg-info-slot egg-info 'components)
        (filter-map (lambda (c)
                      (and (pair? c)
                           (pair? (cdr c))
                           (eq? (car c) type)
                           (cadr c)))
                    (egg-info-slot egg-info 'components)))))

(define (egg-extensions egg-file)
  (egg-components egg-file 'extension))

(define (egg-programs egg-file)
  (egg-components egg-file 'program))
