;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Generate wiki documentation.
;;;
;;; Copyright (c) 2018, Evan Hanson
;;;
;;; See LICENSE for details.
;;;

(declare
  (module (beaker wiki))
  (export document-dependencies
          document-extension
          document-license
          document-program)
  (import (beaker egg info)
          (chicken io)
          (chicken pathname)
          (chicken port)
          (chicken process)
          (chicken process-context)
          (chicken read-syntax)
          (chicken string)
          (markdown-svnwiki)
          (schematic extract)
          (srfi 13)))

;;
;; Reads Scheme source from `input` and writes wiki documentation
;; fragments to `output`.
;;
;; If no arguments are given, `input` and `output` default to
;; the `(current-input-port)` and `(current-output-port)`.
;;
(define (generate-documentation #!optional input output)
  (let* ((buffer  (open-output-string))
         (input*  (or input (current-input-port)))
         (output* (or output (current-output-port)))
         (_       (extract-definitions '(";;") #t input* buffer))
         (buffer* (open-input-string (get-output-string buffer))))
    (parameterize ((current-input-port buffer*)
                   (current-output-port output*))
      (do ((x (read) (read)))
          ((eof-object? x))
        (for-each (lambda (d)
                    (unless (eq? (car d) 'declaration)
                      (print "<" (car d) ">" (cdr d) "</" (car d) ">")))
                  (cdr x))
        (newline)
        (markdown->svnwiki (car x))))))

(define (document-file file)
  (string-trim-both
   (with-input-from-file file
    (lambda ()
      (with-output-to-string generate-documentation)))))

(define (document-command command)
  (string-append " " (string-trim-both (with-input-from-pipe command read-string))))

(define (document-extension file)
  (document-file file))

(define (document-program file command)
  (string-append (document-command command) "\n\n" (document-file file)))

(define (document-dependencies egg-file)
  (string-intersperse
   (map (lambda (d) (conc "* [[/eggref/5/" d "|" d "]]"))
        (egg-dependencies egg-file))
   "\n"))

(define (document-license file)
  (string-intersperse
   (map (lambda (l) (string-append " " l))
        (with-input-from-file file read-lines))
   "\n"))
